<?php
/***
 * Convert Xoops + CBB (newbb) to phpBB
 * 
 * --
 * Alain Reguera Delgado <alain.reguera@gmail.com>
 ***/

class NEWBB_TO_PHPBB
{
    public $db_host;
    public $db_user;
    public $db_pass;
    public $db_conn;
    public $db_phpbb_db;
    public $db_phpbb_tbl;
    public $db_xoops_db;
    public $db_xoops_tbl;


   /***
    * Class Construct
    * ----------------------------------------------------
    ***/

    function __construct()
    {
        // Initialize DB variables
        $this->db_host        = DB_HOST;
        $this->db_user        = DB_USER;
        $this->db_pass        = DB_PASS;
        $this->db_phpbb_db    = DB_PHPBB_DB;
        $this->db_phpbb_tbl   = DB_PHPBB_TBL;
        $this->db_xoops_db    = DB_XOOPS_DB;
        $this->db_xoops_tbl   = DB_XOOPS_TBL;

        // Connect to MySQL database
        $this->db_conn = mysql_connect($this->db_host, $this->db_user, $this->db_pass);
    }

   /***
    * Where is my position in the migration ?
    * ----------------------------------------------------
    * It is somehow a breadcrumb of where you are in the migration process.
    */

    function get_stepPosition()
    {
        $html = '';

        // Define migration process stepts
        $steps = array(0 => 'Configuration',
                       1 => 'Copying data', 
                       2 => 'Updating LDAP userPassword');

        $position = isset($_POST['step'])?$_POST['step']:0;
   
        foreach ( $steps as $key => $value )
        {
            if ( $position == $key)
            {
                $html .= '<strong>'.$key . '. ' . $value.'</strong>' . ' | '; 
            }
            else
            {
                $html .= $key .'. '.$value . ' | ' ;
            }
        }

        $html = 'Steps: ' . $html;

        return $html;
    }

   /***
    * Configuration Form
    * ----------------------------------------------------
    * Show the configuration values.
    ***/

    function get_configForm()
    {
	    $html = '<form name="" action="" method="post">';

        $html .= '<h3>LDAP configuration</h3>';
        $html .= '<ul>';
        $html .= '<li class="description">LDAP Server: </li>';
        $html .= '<li class="value">'.LDAP_HOST.'</li>';
        $html .= '<li class="description">LDAP Server Port: </li>';
        $html .= '<li class="value">'.LDAP_PORT.'</li>';
        $html .= '<li class="description">LDAP bind Root DN: </li>';
        $html .= '<li class="value">'.LDAP_ROOTDN.'</li>';
        $html .= '<li class="description">LDAP user base DN: </li>';
        $html .= '<li class="value">'.LDAP_BASEDN.'</li>';
        $html .= '<li class="description">LDAP bind Root Password: </li>';
		$html .= '<li class="value">*****************</li>';
        $html .= '<li class="description">LDAP userPassword schema: </li>';
        $html .= '<li class="value">'. LDAP_AUTHSCHEMA .'</li>';
        $html .= '</ul>';

	    $html .= '<h3>Common DB configuration:</h3>';
	    $html .= '<ul>';
		$html .= '<li class="description">Server: </li>';
		$html .= '<li class="value">'.DB_HOST.'</li>';
		$html .= '<li class="description">Username:</li>';
		$html .= '<li class="value">'.DB_USER.'</li>';
		$html .= '<li class="description">Password:</li>';
		$html .= '<li class="value">*****************</li>';
        $html .= '</ul>';

	    $html .= '<h3>Xoops configuration:</h3>';
	    $html .= '<ul>';
		$html .= '<li class="description">Xoops database name:</li>';
		$html .= '<li class="value">'.DB_XOOPS_DB.'</li>';
		$html .= '<li class="description">Xoops table prefix:</li>';
		$html .= '<li class="value">'.DB_XOOPS_TBL.'</li>';
	    $html .= '</ul>';

	    $html .= '<h3>phpBB configuration:</h3>';
	    $html .= '<ul>';
		$html .= '<li class="description">Phpbb database name:</li>';
		$html .= '<li class="value">'.DB_PHPBB_DB.'</li>';
		$html .= '<li class="description">Phpbb table prefix:</li>';
		$html .= '<li class="value">'.DB_PHPBB_TBL.'</li>';
	    $html .= '</ul>';

        $html .= '<h3>Mail configuration</h3>';
	    $html .= '<ul>';
		$html .= '<li class="description">Send notifications by email:</li>';
		$html .= '<li class="value">'.MAIL_NOTIFICATION.'</li>';
	    $html .= '</ul>';

        $html .= '<div class="right">';
	    $html .= '<input type="hidden" value="1" name="step" />';
	    $html .= '<input type="submit" value="Next" name="action" id="Next" />';
        $html .= '</div>';
	    $html .= '</form>';

        return $html;
    }

   /***
    * Groups
    * ----------------------------------------------------
    * All users in xoops.users will be inserted into phpBB.users
    * using the REGISTERED group (group_id = 2). Forums administrators should
    * be redifined after migration.
    ***/

   /*** 
    * Users 
    * ---------------------------------------------------- 
    * Basic fields are copied from xoops.users to phpBB.users.
    *
    * Password field should be redifined by the user in order to get logged in
    * after the migration.
    *
    * If LDAP authentication is used the directory structure should be design
    * to receive uid and userPassword attributes. In this case the migration
    * should be focused from xoops.users to LDAP directory not phpBB.users.
    *
    * The LDAP registration process is (as my understanding): 1. Add an entry
    * for the user in the LDAP directory.  2. Add an entry for the user in the
    * DB (this is automatically done by phpBB). This is needed to relate user
    * against user specific information like topics, posts, etc.
    *
    * As we are using LDAP server for users. This function use php's ldap
    * extension to add users into LDAP directory. If the user do no exist in
    * the Database but in LDAP server, phpBB will automatically insert a
    * record for that user in the phpBB.user table. It is needed to relate
    * user identity to posts, topics, etc .
    *
    * User passwords need to be reseted and a notification could be send to each
    * user telling the new password set. This is requiered because the
    * password codification used in newbb, phpbb and LDAP is different.
    *
    * The structure of LDAP user entries was built with rfc2377 in mind.
    ***/

    function copy_Users()
    {
        global $ldap;

        $html = '<h3>Users</h3>';

        // Remove phpBB.users. Number 52 seems to be the greatest user_id
        // value when no user has been created.
        $html .= '<li class="description">Cleanning up users ... </li>';
        $sql = sprintf('DELETE FROM %s.%susers WHERE user_id > 52;', $this->db_phpbb_db, $this->db_phpbb_tbl);
        mysql_query( $sql, $this->db_conn ) or die ( mysql_error( $this->db_conn ) );
        $html .= '<li class="value">OK</li>'; 

    	// Add users into LDAP directory
        $html .= '<li class="description">Copying users into LDAP directory ... </li>';
    	$sql = sprintf("SELECT uname, 
                               name, 
                               email, 
                               pass
                               FROM %s.%susers WHERE uid > 1", $this->db_xoops_db, $this->db_xoops_tbl);

        $result = mysql_query( $sql, $this->db_conn ) or die ( mysql_error() );

		$counter = 0;

		while ( $entry = mysql_fetch_array( $result ))
		{
            // Add xoops.users into LDAP directory
            if ( $ldap->add_User( $entry ) === true )
            {
                $counter++;
            }
        }
        
        $html .= '<li class="value">'. $counter .' record(s) copied successfully.</li>';

        return $html;
    }

   /***
    * Categories 
    * ----------------------------------------------------
    * Not copied. In phpBB there is no category.
    */


   /***
    * Copy Forum
    * ----------------------------------------------------
    */

    function copy_Forums()
    {
        $html = '<h3>Forums</h3>';

        // Clean up Forums
        $html .= '<li class="description">Cleanning up forum structure ... </li>';
        $sql = sprintf("TRUNCATE %s.%sforums;", $this->db_phpbb_db, $this->db_phpbb_tbl);
        mysql_query( $sql, $this->db_conn) or die( mysql_error($this->db_conn));
        $html .= '<li class="value">OK</li>'; 

        // Copy Forums
        $html .= '<li class="description">Copying forums ... </li>';
        $sql = sprintf("INSERT INTO %s.%sforums (forum_name,
                                                 forum_desc,
                                                 forum_topics,
                                                 forum_topics_real,
                                                 forum_type,
                                                 forum_posts) SELECT forum_name,
                                                                    forum_desc,
                                                                    forum_topics,
                                                                    forum_topics,
                                                                    (SELECT 1 AS forum_type),
                                                                    forum_posts FROM %s.%sbb_forums;",
                                    $this->db_phpbb_db, $this->db_phpbb_tbl, 
                                    $this->db_xoops_db, $this->db_xoops_tbl);
        mysql_query( $sql, $this->db_conn ) or die ( mysql_error( $this->db_conn ) );

        // Get forums recently added to update order, ownership, ant type. 
        $sql = sprintf('SELECT forum_id, left_id, right_id FROM %s.%sforums;', 
                                    $this->db_phpbb_db, $this->db_phpbb_tbl );
        $result = mysql_query( $sql, $this->db_conn );

        // Sanitize forum's order one by one
        $left_id = 1;
        $right_id = 2;
        $counter = 0;
        while ( $row = mysql_fetch_array( $result ) )
        {
            $sql = sprintf("UPDATE %s.%sforums SET left_id = %d, right_id = %d WHERE forum_id = %d;", 
                                $this->db_phpbb_db, $this->db_phpbb_tbl, $left_id, $right_id, $row['forum_id']);
            mysql_query( $sql ) or die ( mysql_error( $this->db_conn ) );

            $left_id  = $left_id + 2;
            $right_id = $left_id + 1;

            // Counter
            $counter++;
        }

        $html .= '<li class="value">'.$counter.' record(s) copied successfully.</li>'; 

        return $html;
    }

   /*** 
    * Topics
    * ----------------------------------------------------
    * When doing Topic copying the user used will be administrator. This means
    * that all topics and posts after migration will be own by the
    * administrator user. 
    ***/

    function copy_Topics()
    {
        $html = '<h3>Topics</h3>';

        // Claen up topics
        $html .= '<li class="description">Cleanning up topics ... </li>';
        $sql = sprintf("TRUNCATE %s.%stopics;", $this->db_phpbb_db, $this->db_phpbb_tbl);
        mysql_query($sql, $this->db_conn) or die( mysql_error($this->db_conn));
        $html .= '<li class="value">OK</li>'; 

        // Copy topics
        $html .= '<li class="description">Copying topics ... </li>';
        $sql = sprintf("INSERT INTO %s.%stopics (forum_id, 
                                                 topic_title, 
                                                 topic_time, 
                                                 topic_last_post_time,
                                                 topic_views, 
                                                 topic_last_poster_id,
                                                 topic_poster,
                                                 topic_replies,
                                                 topic_replies_real) SELECT forum_id, 
                                                          topic_title, 
                                                          topic_time, 
                                                          topic_time, 
                                                          topic_views, 
                                                          (SELECT 2 AS last_poster_id),
                                                          (SELECT 2 AS last_poster_id),
                                                          topic_replies, 
                                                          topic_replies 
                                                          FROM %s.%sbb_topics",
                                $this->db_phpbb_db, $this->db_phpbb_tbl, 
                                $this->db_xoops_db, $this->db_xoops_tbl);

        mysql_query($sql, $this->db_conn) or die( mysql_error($this->db_conn));
        $html .= '<li class="value">'.mysql_affected_rows().' record(s) copied successfully.</li>'; 

        return $html;
    }

   /***
    * Posts
    * ----------------------------------------------------
    ***/

    function copy_Posts()
    {
        $html = '<h3>Posts</h3>';
        
        // Clean up posts
        $html .= '<li class="description">Cleanning up posts ... </li>';
        $sql = sprintf("TRUNCATE %s.%sposts;", $this->db_phpbb_db, $this->db_phpbb_tbl);
        mysql_query($sql, $this->db_conn) or die( mysql_error($this->db_conn));
        $html .= '<li class="value">OK</li>'; 

        $html .= '<li class="description">Copying topics ... </li>';
        $sql = sprintf("INSERT INTO %s.%sposts (topic_id,
                                                forum_id,
                                                poster_id,
                                                post_time,
                                                post_subject,
                                                post_text) SELECT t1.topic_id,
                                                                  t1.forum_id,
                                                                  (SELECT 2 AS poster_id),
                                                                  t1.post_time,
                                                                  t1.subject,
                                                                  t2.post_text
                                                                  FROM %s.%sbb_posts t1 
                                                                  LEFT JOIN %s.%sbb_posts_text t2 
                                                                  ON t2.post_id = t1.post_id;",
                                            $this->db_phpbb_db, $this->db_phpbb_tbl,
                                            $this->db_xoops_db, $this->db_xoops_tbl,
                                            $this->db_xoops_db, $this->db_xoops_tbl);

        mysql_query($sql, $this->db_conn) or die( mysql_error($this->db_conn));
        $html .= '<li class="value">'.mysql_affected_rows().' record(s) copied successfully.</li>'; 

        return $html;

    }

   /***
    * Generate random password
    * ----------------------------------------------------
    ***/

    function get_randomPass()
    {
        // Use lower case letters
        $seed_alpha_lower = array('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
                      'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'x', 'y', 'z');

        // Use upper case letters
        $seed_alpha_upper = array();
        foreach ( $seed_alpha_lower as $key => $value )
        {
            $seed_alpha_upper[$key] = strtoupper($value);
        }

        // Use numbers
        $seed_nums = array('1', '2', '3', '4', '5', '6', '7', '8', '9', '0');

        // Use some symbols chars
        $seed_chars = array('!', '@', '#', '$', '=');

        // Make a big array with all the previous chars
        $seed = array_merge($seed_alpha_lower, $seed_alpha_upper, $seed_chars, $seed_nums);

        // Build password
        $userPassword = '';
        $passwordLength = 8;
        for ($i = 0; $i < $passwordLength; $i++)
        {
            $userPassword = $userPassword . $seed[array_rand($seed)];
        }

        return $userPassword;
    }

   /***
    * Class Destruct
    * ----------------------------------------------------
    ***/

    function __destruct()
    {
        mysql_close( $this->db_conn );
    }
}

$newbb_to_phpbb = new NEWBB_TO_PHPBB;
